from drstats.devrant import Devrant
from drstats.db import get_db
import json
import asyncio
from pprint import pprint as pp
import requests

dr = Devrant()
db = get_db()


def plain_object(obj):
    for key, value in obj.items():
        if isinstance(value, list) or isinstance(value, dict):
            value = json.dumps(value)
            obj[key] = value
    return obj


from datetime import datetime


def timestamp_to_string(timestamp):
    return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d %H:%M:%S")


async def get_recent_rants(start_from=1, page_size=10):
    page = 0
    while True:
        rants = dr.get_rants("recent", page_size, start_from)["rants"]
        page += 1
        for rant in rants:
            if rant is None:
                break
            rant["tags"] = json.dumps("tags" in rant and rant["tags"] or "")
            rant["created"] = timestamp_to_string(rant["created_time"])
            rant = plain_object(rant)

            yield rant
        start_from += page_size


async def sync_rants():
    count = 0
    start_from = 0

    page_size = 20

    try:
        async for rant in get_recent_rants(start_from, page_size):
            start_from += page_size
            count += 1
            rant["tags"] = json.dumps(rant["tags"])
            db["rants"].upsert(rant, ["id"])
            print(f"Upserted {count} rant(s).")
    except:
        print("Rate limit of server exceeded. That's normal.s")


async def sync_comments():
    comments_synced = 0
    rants_synced = 0
    for rant in db["rants"].find(order_by="-id"):
        rants_synced += 1
        comments = dr.get_rant(rant["id"])["comments"]
        for comment in comments:
            comments_synced += 1
            comment["created"] = timestamp_to_string(comment["created_time"])
            comment = plain_object(comment)
            comment["rant_id"] = rant["id"]

            db["comments"].upsert(comment, ["id"])
            print(f"Synced {rants_synced} rants with {comments_synced} comments.")


def sync():
    print(asyncio.run(sync_comments()))
    print(asyncio.run(sync_rants()))
